@@ -26,16 +26,10 @@ checksum_algo_name = checksum_algo.__name__.replace('openssl_','')
 import hmac
 import struct
 import threading
+from binascii import hexlify, unhexlify
 
-try:
-    import cPickle
-except ImportError:
-    import pickle as cPickle  # lint:ok
-
-try:
-    from cStringIO import StringIO
-except ImportError:
-    from StringIO import StringIO  # lint:ok
+import pickle as cPickle  # lint:ok
+from io import BytesIO  # lint:ok
 
 class SecurePickler(object):
     def __init__(self, checksum_key, file, *p, **kw):
@@ -51,7 +45,7 @@ class SecurePickler(object):
         try:
             return self.local.buf
         except AttributeError:
-            self.local.buf = buf = StringIO()
+            self.local.buf = buf = BytesIO()
             return buf
 
     @buf.deleter
@@ -83,12 +77,12 @@ class SecurePickler(object):
         # dump to underlying pickler, then pick up the results
         self.pickler.dump(val)
         rv = self.buf.getvalue()
-        self.buf.reset()
+        self.buf.seek(0)
         self.buf.truncate()
 
         # compute HMAC, and prepend to output
-        md = hmac.HMAC(self.checksum_key, rv, checksum_algo).hexdigest()
-        self.file.write(struct.pack('<L',len(rv)).encode("hex"))
+        md = hmac.HMAC(self.checksum_key, rv, checksum_algo).hexdigest().encode("ascii")
+        self.file.write(hexlify(struct.pack('<L',len(rv))))
         self.file.write(md)
         self.file.write(rv)
 
@@ -106,7 +100,7 @@ class SecureUnpickler(object):
         try:
             return self.local.buf
         except AttributeError:
-            self.local.buf = buf = StringIO()
+            self.local.buf = buf = BytesIO()
             return buf
 
     @buf.deleter
@@ -134,11 +128,11 @@ class SecureUnpickler(object):
     def persistent_load(self, value):  # lint:ok
         self.unpickler.persistent_load = value
 
-    def load(self, headlen = len(struct.pack('<L',0).encode("hex"))):
+    def load(self, headlen = len(struct.pack('<L',0)) * 2):
         datalen = self.file.read(headlen)
         if not datalen:
-            raise EOFError, "Cannot read secure packet header"
-        datalen, = struct.unpack('<L', datalen.decode("hex") )
+            raise EOFError("Cannot read secure packet header")
+        datalen, = struct.unpack('<L', unhexlify(datalen) )
 
         ref_md = hmac.HMAC(self.checksum_key, None, checksum_algo)
         md = self.file.read(ref_md.digest_size*2)
@@ -146,17 +140,17 @@ class SecureUnpickler(object):
         data = self.file.read(datalen)
         ref_md.update(data)
 
-        ref_md = ref_md.hexdigest()
+        ref_md = ref_md.hexdigest().encode("ascii")
         if ref_md != md:
-            raise ValueError, "MAC mismatch unpickling"
+            raise ValueError("MAC mismatch unpickling")
 
         buf = self.buf
-        buf.reset()
+        buf.seek(0)
         buf.write(data)
         buf.truncate()
-        buf.reset()
+        buf.seek(0)
         rv = self.unpickler.load()
-        buf.reset()
+        buf.seek(0)
         buf.truncate()
         return rv
 
@@ -165,7 +159,7 @@ def dump(key, obj, file, *p, **kw):
     pickler.dump(obj)
 
 def dumps(key, obj, *p, **kw):
-    buf = StringIO()
+    buf = BytesIO()
     pickler = SecurePickler(key, buf, *p, **kw)
     pickler.dump(obj)
     return buf.getvalue()
@@ -175,6 +169,6 @@ def load(key, file, *p, **kw):
     return unpickler.load()
 
 def loads(key, str, *p, **kw):
-    buf = StringIO(str)
+    buf = BytesIO(str)
     unpickler = SecureUnpickler(key, buf, *p, **kw)
     return unpickler.load()
